@@ -21,7 +21,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <libHX/arbtree.h>
 #include <libHX/ctype_helper.h>
 #include <libHX/defs.h>
 #include <libHX/deque.h>
@@ -31,9 +30,7 @@
 #include "pam_mount.h"
 
 /* Functions */
-static int fstype_nodev(const char *);
 static inline bool mkmountpoint(struct vol *, const char *);
-static void run_ofl(const struct config * const, struct HXbtree *);
 
 //-----------------------------------------------------------------------------
 /**
@@ -49,7 +46,7 @@ static void run_ofl(const struct config * const, struct HXbtree *);
  */
 static void log_output(int fd, const char *cmsg)
 {
-	char buf[BUFSIZ + 1];
+	hxmc_t *buf = NULL;
 	FILE *fp;
 
 	if ((fp = fdopen(fd, "r")) == NULL) {
@@ -60,7 +57,7 @@ static void log_output(int fd, const char *cmsg)
 
 	setvbuf(fp, NULL, _IOLBF, 0);
 	do {
-		if (fgets(buf, sizeof(buf), fp) == NULL)
+		if (HX_getl(&buf, fp) == NULL)
 			break;
 		HX_chomp(buf);
 		if (*buf != '\0' && cmsg != NULL) {
@@ -71,6 +68,7 @@ static void log_output(int fd, const char *cmsg)
 		l0g("%s\n", buf);
 	} while (true);
 	fclose(fp);
+	HXmc_free(buf);
 }
 
 /**
@@ -81,16 +79,18 @@ static void log_output(int fd, const char *cmsg)
  * Runs `ofl` on a directory/mountpoint and logs its output, for debugging
  * purposes.
  */
-static void run_ofl(const struct config *const config, struct HXbtree *vinfo)
+static void run_ofl(const struct config *const config,
+    struct HXformat_map *vinfo)
 {
-	const char *mntpt;
+	hxmc_t *mntpt = NULL;
 	struct stat sb;
 
-	mntpt = HXbtree_get(vinfo, "MNTPT");
-	if (stat(mntpt, &sb) < 0 && errno == ENOENT)
-		return;
-	ofl_printf = misc_warn;
-	ofl(mntpt, 0);
+	HXformat_aprintf(vinfo, &mntpt, "%(MNTPT)");
+	if (!(stat(mntpt, &sb) < 0 && errno == ENOENT)) {
+		ofl_printf = misc_warn;
+		ofl(mntpt, 0);
+	}
+	HXmc_free(mntpt);
 }
 
 /**
@@ -108,7 +108,7 @@ static void run_ofl(const struct config *const config, struct HXbtree *vinfo)
 	/* elsewhere */
 #else
 int pmt_already_mounted(const struct config *const config,
-    const struct vol *vpt, struct HXbtree *vinfo)
+    const struct vol *vpt, struct HXformat_map *vinfo)
 {
 	l0g("check for previous mount not implemented on arch.\n");
 	return -1;
@@ -305,7 +305,7 @@ static inline bool mkmountpoint(struct vol *volume, const char *d)
  * Returns zero on error, positive non-zero for success.
  */
 int do_unmount(const struct config *config, struct vol *vpt,
-    struct HXbtree *vinfo, const char *const password)
+    struct HXformat_map *vinfo, const char *const password)
 {
 	struct HXdeque *argv;
 	struct HXproc proc;
@@ -365,7 +365,7 @@ int do_unmount(const struct config *config, struct vol *vpt,
 }
 
 static int check_filesystem(const struct config *config, const struct vol *vpt,
-    struct HXbtree *vinfo)
+    struct HXformat_map *vinfo)
 {
 /* PRE:    config points to a valid struct config
  *         config->volume[vol] is a valid struct vol
@@ -434,7 +434,7 @@ static int check_filesystem(const struct config *config, const struct vol *vpt,
  * @vinfo:	variable substituions
  */
 static void mount_set_fsck(const struct config *config,
-    const struct vol *vol, struct HXbtree *vinfo)
+    const struct vol *vol, struct HXformat_map *vinfo)
 {
 	const struct HXdeque_node *i;
 	hxmc_t *string, *current;
@@ -462,12 +462,12 @@ static void mount_set_fsck(const struct config *config,
  * @config:	current config
  * @vpt:	volume descriptor
  * @vinfo:
- * @password:	login password
+ * @password:	login password (may be %NULL)
  *
  * Returns zero on error, positive non-zero for success.
  */
 int do_mount(const struct config *config, struct vol *vpt,
-    struct HXbtree *vinfo, const char *password)
+    struct HXformat_map *vinfo, const char *password)
 {
 	const struct HXdeque_node *n;
 	struct HXdeque *argv;
@@ -476,7 +476,6 @@ int do_mount(const struct config *config, struct vol *vpt,
 	int ret;
 
 	assert(vinfo != NULL);
-	assert(password != NULL);
 
 	ret = pmt_already_mounted(config, vpt, vinfo);
 	if (ret < 0) {
@@ -575,7 +574,7 @@ int mount_op(mount_op_fn_t *mnt, const struct config *config,
     struct vol *vpt, const char *password)
 {
 	int fnval;
-	struct HXbtree *vinfo;
+	struct HXformat_map *vinfo;
 	struct passwd *pe;
 	hxmc_t *options;
 	char real_mpt[PATH_MAX+1];
@@ -643,7 +642,8 @@ int mount_op(mount_op_fn_t *mnt, const struct config *config,
  * Returns 1 if the filesystem does not require a block device, 0 if it does
  * require a block device, -1 if we could not find out.
  */
-static int fstype_nodev(const char *name) {
+int fstype_nodev(const char *name)
+{
 	char buf[80];
 	FILE *fp;
 
