@@ -1,7 +1,7 @@
 /*
  *	pam_mount
  *	Copyright (C) Elvis Pfützenreuter <epx@conectiva.com>, 2000
- *	Copyright © Jan Engelhardt, 2005 - 2008
+ *	Copyright © Jan Engelhardt, 2005 - 2009
  *	Copyright © Bastian Kleineidam, 2005
  *
  *	This file is part of pam_mount; you can redistribute it and/or
@@ -393,7 +393,7 @@ static int modify_pm_count(struct config *config, char *user,
     char *operation)
 {
 	FILE *fp = NULL;
-	struct HXbtree *vinfo;
+	struct HXformat_map *vinfo;
 	struct HXdeque *argv;
 	struct HXproc proc;
 	int ret = -1, use_count;
@@ -435,6 +435,67 @@ static int modify_pm_count(struct config *config, char *user,
 	return ret;
 }
 
+/**
+ * grab_authtok - get the password from PAM
+ */
+static char *grab_authtok(pam_handle_t *pamh)
+{
+	char *system_authtok = NULL;
+	int ret;
+
+	ret = pam_get_data(pamh, "pam_mount_system_authtok",
+	      static_cast(const void **, static_cast(void *, &system_authtok)));
+	if (ret != PAM_SUCCESS) {
+		if (Args.get_pw_interactive) {
+			ret = read_password(pamh, Config.msg_sessionpw,
+			      &system_authtok);
+			if (ret != PAM_SUCCESS)
+				l0g("warning: could not obtain password "
+				    "interactively either\n");
+		}
+		/*
+		 * Proceed without a password. Some volumes may not need one,
+		 * e.g. bind mounts and networked/unencrypted volumes.
+		 */
+	}
+	if (system_authtok == NULL)
+		system_authtok = xstrdup("");
+	return system_authtok;
+}
+
+static int process_volumes(struct config *config, const char *authtok)
+{
+	int ret = PAM_SUCCESS;
+	struct vol *vol;
+
+	HXlist_for_each_entry(vol, &config->volume_list, list) {
+		/*
+		 * Remember what we processed already - the function can
+		 * be called multiple times.
+		 */
+		if (vol->mnt_processed)
+			continue;
+		vol->mnt_processed = true;
+		/*
+		 * luserconf_volume_record_sane() is called here so that a user
+		 * can nest loopback images. otherwise ownership tests will
+		 * fail if parent loopback image not yet mounted.
+		 * volume_record_sane() is here to be consistent.
+		 */
+		if (!volume_record_sane(config, vol))
+			continue;
+		if (!vol->globalconf &&
+		    !luserconf_volume_record_sane(config, vol))
+			continue;
+
+		if (!mount_op(do_mount, config, vol, authtok)) {
+			l0g("mount of %s failed\n", znul(vol->volume));
+			ret = PAM_SERVICE_ERR;
+		}
+	}
+	return ret;
+}
+
 /**
  * pam_sm_open_session -
  * @pamh:	PAM handle
@@ -449,9 +510,8 @@ static int modify_pm_count(struct config *config, char *user,
 PAM_EXTERN EXPORT_SYMBOL int pam_sm_open_session(pam_handle_t *pamh, int flags,
     int argc, const char **argv)
 {
-	struct vol *vol;
 	int ret;
-	unsigned int krb5_set;
+	const char *krb5;
 	char *system_authtok = NULL;
 	const void *tmp;
 	int getval;
@@ -463,12 +523,18 @@ PAM_EXTERN EXPORT_SYMBOL int pam_sm_open_session(pam_handle_t *pamh, int flags,
 
 	w4rn(PACKAGE_STRING ": entering session stage\n");
 
+	/*
+	 * Environment variables set with setenv() only last while PAM is
+	 * active, i.e. disappear when the shell is started. On the other hand,
+	 * variabled fed to pam_putenv() are only visible once the shell
+	 * started.
+	 */
 	/*
 	 * Get the Kerberos CCNAME so we can make it available to the
 	 * mount command later on.
 	 */
-	krb5_set = getenv("KRB5CCNAME") != NULL;
-	if (setenv("KRB5CCNAME", pam_getenv(pamh, "KRB5CCNAME"), 1) < 0)
+	krb5 = pam_getenv(pamh, "KRB5CCNAME");
+	if (krb5 != NULL && setenv("KRB5CCNAME", krb5, true) < 0)
 		l0g("KRB5CCNAME setenv failed\n");
 
 	/* Store initialized config as PAM data */
@@ -482,71 +548,50 @@ PAM_EXTERN EXPORT_SYMBOL int pam_sm_open_session(pam_handle_t *pamh, int flags,
 		}
 	}
 
-	if (Config.luserconf == NULL || strlen(Config.luserconf) == 0)
-		;
-	else if (!pmt_fileop_exists(Config.luserconf))
-		;
-	else if (pmt_fileop_owns(Config.user, Config.luserconf)) {
-		w4rn("going to readconfig %s\n", Config.luserconf);
-		if (!readconfig(Config.luserconf, false, &Config)) {
-			ret = PAM_SERVICE_ERR;
-			goto out;
-		}
-	} else
-		w4rn("%s does not exist or is not owned by user\n",
-		     Config.luserconf);
-	if (Config.volume_list.items == 0) {
-		w4rn("no volumes to mount\n");
-		ret = PAM_SUCCESS;
-		goto out;
-	}
 	if (!expandconfig(&Config)) {
 		l0g("error expanding configuration\n");
 		ret = PAM_SERVICE_ERR;
 		goto out;
 	}
-
-	ret = pam_get_data(pamh, "pam_mount_system_authtok",
-	      static_cast(const void **, static_cast(void *, &system_authtok)));
-	if (ret != PAM_SUCCESS) {
-		if (Args.get_pw_interactive) {
-			ret = read_password(pamh, Config.msg_sessionpw, &system_authtok);
-			if (ret != PAM_SUCCESS)
-				l0g("warning: could not obtain password "
-				    "interactively either\n");
-		}
-		/*
-		 * Proceed without a password. Some volumes may not need one,
-		 * e.g. bind mounts and networked/unencrypted volumes.
-		 */
-	}
-	if (system_authtok == NULL)
-		system_authtok = xstrdup("");
+	if (Config.volume_list.items > 0)
+		/* There are some volumes, so grab a password. */
+		system_authtok = grab_authtok(pamh);
 
 	misc_dump_id("Session open");
-
 	envpath_init(Config.path);
-	HXlist_for_each_entry(vol, &Config.volume_list, list) {
-		/*
-		 * luserconf_volume_record_sane() is called here so that a user
-		 * can nest loopback images. otherwise ownership tests will
-		 * fail if parent loopback image not yet mounted. 
-		 * volume_record_sane() is here to be consistent.
-		 */
-		if (!volume_record_sane(&Config, vol))
-			continue;
-		if (!vol->globalconf &&
-		    !luserconf_volume_record_sane(&Config, vol))
-			continue;
+	ret = process_volumes(&Config, system_authtok);
 
-		if (!mount_op(do_mount, &Config, vol, system_authtok)) {
-			l0g("mount of %s failed\n", znul(vol->volume));
+	/*
+	 * Read luserconf after mounting of initial volumes. This makes it
+	 * possible to store luserconfs on net volumes themselves.
+	 */
+	if (Config.luserconf != NULL && *Config.luserconf != '\0' &&
+	    pmt_fileop_exists(Config.luserconf)) {
+		w4rn("going to readconfig %s\n", Config.luserconf);
+		if (!pmt_fileop_owns(Config.user, Config.luserconf)) {
+			w4rn("%s does not exist or is not owned by user\n",
+			     Config.luserconf);
+		} else if (!readconfig(Config.luserconf, false, &Config)) {
 			ret = PAM_SERVICE_ERR;
+		} else if (!expandconfig(&Config)) {
+			ret = PAM_SERVICE_ERR;
+			l0g("error expanding configuration\n");
 		}
 	}
-	memset(system_authtok, 0, strlen(system_authtok));
-	if (krb5_set)
-		unsetenv("KRB5CCNAME");
+
+	if (Config.volume_list.items == 0) {
+		w4rn("no volumes to mount\n");
+		ret = PAM_SUCCESS;
+	} else {
+		if (system_authtok == NULL)
+			system_authtok = grab_authtok(pamh);
+		ret = process_volumes(&Config, system_authtok);
+	}
+
+	if (system_authtok != NULL) {
+		memset(system_authtok, 0, strlen(system_authtok));
+		free(system_authtok);
+	}
 	modify_pm_count(&Config, Config.user, "1");
 	envpath_restore();
 	if (getuid() == 0)
@@ -564,6 +609,8 @@ PAM_EXTERN EXPORT_SYMBOL int pam_sm_open_session(pam_handle_t *pamh, int flags,
 	 */
 	ret = PAM_SUCCESS;
  out:
+	if (krb5 != NULL)
+		unsetenv("KRB5CCNAME");
 	w4rn("done opening session (ret=%d)\n", ret);
 	common_exit();
 	return ret;
