@@ -4,7 +4,6 @@
 #include <sys/types.h>
 #include <limits.h>
 #include <stdbool.h>
-#include <libHX/clist.h>
 #include <libHX/list.h>
 #include <libHX/option.h>
 #include <libHX/string.h>
@@ -42,8 +41,8 @@
 	misc_warn(("%s(%s:%u): " fmt), pmtlog_prefix, HX_basename(__FILE__), \
 	__LINE__, ## __VA_ARGS__)
 
-struct HXbtree;
 struct HXdeque;
+struct HXformatmap;
 struct HXproc;
 struct loop_info64;
 
@@ -94,6 +93,10 @@ struct vol {
 	bool globalconf;
 	/* set, so that umount can rmdir it */
 	bool created_mntpt;
+	/* expansion already took place */
+	bool is_expanded;
+	/* was handed off to mount_op() */
+	bool mnt_processed;
 	const char *user;
 	char *fstype, *server, *volume, *mountpoint, *cipher;
 	char *fs_key_cipher, *fs_key_hash, *fs_key_path;
@@ -119,7 +122,7 @@ struct config {
 	bool seen_mntoptions_require, seen_mntoptions_allow;
 	hxmc_t *luserconf;
 	struct HXdeque *command[_CMD_MAX];
-	struct HXbtree *options_require, *options_allow, *options_deny;
+	struct HXmap *options_require, *options_allow, *options_deny;
 	struct HXclist_head volume_list;
 	int level;
 	char *msg_authpw, *msg_sessionpw, *path;
@@ -175,12 +178,12 @@ struct ehd_crypto_ops {
 };
 
 typedef int (mount_op_fn_t)(const struct config *, struct vol *,
-	struct HXbtree *, const char *);
+	struct HXformat_map *, const char *);
 
 /*
  *
  */
-static inline void format_add(struct HXbtree *table, const char *key,
+static inline void format_add(struct HXformat_map *table, const char *key,
     const char *value)
 {
 	if (value == NULL)
@@ -233,16 +236,16 @@ extern int misc_warn(const char *, ...);
  *	MISC.C
  */
 extern void arglist_add(struct HXdeque *, const char *,
-	const struct HXbtree *);
+	const struct HXformat_map *);
 extern struct HXdeque *arglist_build(const struct HXdeque *,
-	const struct HXbtree *);
+	const struct HXformat_map *);
 extern void arglist_log(const struct HXdeque *);
 extern void arglist_llog(const char *const *);
 extern bool kvplist_contains(const struct HXclist_head *, const char *);
 extern char *kvplist_get(const struct HXclist_head *, const char *);
 extern void kvplist_genocide(struct HXclist_head *);
 extern hxmc_t *kvplist_to_str(const struct HXclist_head *);
-extern void misc_add_ntdom(struct HXbtree *, const char *);
+extern void misc_add_ntdom(struct HXformat_map *, const char *);
 extern void misc_dump_id(const char *);
 extern int pmt_fileop_exists(const char *);
 extern int pmt_fileop_owns(const char *, const char *);
@@ -286,11 +289,12 @@ extern int pmt_cmtab_mounted(const char *, const char *);
  *	MOUNT.C
  */
 extern mount_op_fn_t do_mount, do_unmount;
+extern int fstype_nodev(const char *);
 extern int mount_op(mount_op_fn_t *, const struct config *, struct vol *,
 	const char *);
 extern void umount_final(struct config *);
 extern int pmt_already_mounted(const struct config *,
-	const struct vol *, struct HXbtree *);
+	const struct vol *, struct HXformat_map *);
 extern hxmc_t *pmt_vol_to_dev(const struct vol *);
 extern bool fstype_icase(const char *);
 extern bool fstype2_icase(enum command_type);
