@@ -1,5 +1,5 @@
 /*
- *	Copyright © Jan Engelhardt, 2006 - 2008
+ *	Copyright © Jan Engelhardt, 2006 - 2009
  *
  *	This file is part of pam_mount; you can redistribute it and/or
  *	modify it under the terms of the GNU Lesser General Public License
@@ -19,10 +19,10 @@
 #elif defined(__linux__)
 #	include <mntent.h>
 #endif
-#include <libHX/arbtree.h>
 #include <libHX/ctype_helper.h>
 #include <libHX/defs.h>
 #include <libHX/deque.h>
+#include <libHX/map.h>
 #include <libHX/option.h>
 #include <libHX/string.h>
 #include <libHX/libxml_helper.h>
@@ -52,8 +52,7 @@ enum {
 };
 
 enum {
-	OPT_TREE_FLAGS =
-		HXBT_MAP | HXBT_CKEY | HXBT_SCMP | HXBT_CID,
+	OPT_MAP_FLAGS = HXMAP_SCKEY,
 };
 
 struct callbackmap {
@@ -126,7 +125,7 @@ static bool expand_home(const char *user, char **path_pptr)
  * moment due to to-XML transition period.)
  */
 static bool expand_user(const char *user, char **dest_pptr,
-    const struct HXbtree *vinfo)
+    const struct HXformat_map *vinfo)
 {
 	hxmc_t *tmp = NULL;
 
@@ -149,7 +148,7 @@ static bool expand_user(const char *user, char **dest_pptr,
 bool expandconfig(const struct config *config)
 {
 	const char *u = config->user;
-	struct HXbtree *vinfo;
+	struct HXformat_map *vinfo;
 	struct passwd *pe;
 	struct group *ge;
 	struct kvp *kvp;
@@ -172,6 +171,9 @@ bool expandconfig(const struct config *config)
 	misc_add_ntdom(vinfo, u);
 
 	HXlist_for_each_entry(vpt, &config->volume_list, list) {
+		if (vpt->is_expanded)
+			continue;
+		vpt->is_expanded = true;
 		if (!expand_user(u, &vpt->server, vinfo) ||
 		    !expand_home(u, &vpt->volume) ||
 		    !expand_user(u, &vpt->volume, vinfo) ||
@@ -249,9 +251,9 @@ void freeconfig(struct config *config)
 	HXlist_for_each_entry_safe(vol, next, &config->volume_list, list)
 		volume_free(vol);
 
-	HXbtree_free(config->options_allow);
-	HXbtree_free(config->options_require);
-	HXbtree_free(config->options_deny);
+	HXmap_free(config->options_allow);
+	HXmap_free(config->options_require);
+	HXmap_free(config->options_deny);
 	free(config->user);
 	free(config->msg_authpw);
 	free(config->msg_sessionpw);
@@ -306,7 +308,7 @@ static bool str_to_optkv(struct HXclist_head *optlist, char *str)
 	return false;
 }
 
-static bool str_to_optlist(struct HXbtree *optlist, char *str)
+static bool str_to_optlist(struct HXmap *optlist, char *str)
 {
 	char *value, *ptr;
 
@@ -317,9 +319,9 @@ static bool str_to_optlist(struct HXbtree *optlist, char *str)
 		value = strchr(ptr, '=');
 		if (value != NULL) {
 			*value++ = '\0';
-			HXbtree_add(optlist, ptr, value);
+			HXmap_add(optlist, ptr, value);
 		} else {
-			HXbtree_add(optlist, ptr, NULL);
+			HXmap_add(optlist, ptr, NULL);
 		}
 	}
 
@@ -360,9 +362,9 @@ void initconfig(struct config *config)
 			HXdeque_push(cmd, xstrdup(default_command[i].def[j]));
 	}
 
-	config->options_allow   = HXbtree_init(OPT_TREE_FLAGS);
-	config->options_require = HXbtree_init(OPT_TREE_FLAGS);
-	config->options_deny    = HXbtree_init(OPT_TREE_FLAGS);
+	config->options_allow   = HXmap_init(HXMAPT_DEFAULT, OPT_MAP_FLAGS);
+	config->options_require = HXmap_init(HXMAPT_DEFAULT, OPT_MAP_FLAGS);
+	config->options_deny    = HXmap_init(HXMAPT_DEFAULT, OPT_MAP_FLAGS);
 	str_to_optlist(config->options_allow, options_allow);
 	str_to_optlist(config->options_require, options_require);
 	HXclist_init(&config->volume_list);
@@ -715,8 +717,9 @@ static const char *rc_mntoptions(xmlNode *node, struct config *config,
 
 	if ((options = xml_getprop(node, "allow")) != NULL) {
 		if (!config->seen_mntoptions_allow) {
-			HXbtree_free(config->options_allow);
-			config->options_allow = HXbtree_init(OPT_TREE_FLAGS);
+			HXmap_free(config->options_allow);
+			config->options_allow =
+				HXmap_init(HXMAPT_DEFAULT, OPT_MAP_FLAGS);
 			config->seen_mntoptions_allow = true;
 		}
 		ret = str_to_optlist(config->options_allow, options);
@@ -739,8 +742,9 @@ static const char *rc_mntoptions(xmlNode *node, struct config *config,
 		 * "require"s shall be cumulative.
 		 */
 		if (!config->seen_mntoptions_require) {
-			HXbtree_free(config->options_require);
-			config->options_require = HXbtree_init(OPT_TREE_FLAGS);
+			HXmap_free(config->options_require);
+			config->options_require =
+				HXmap_init(HXMAPT_DEFAULT, OPT_MAP_FLAGS);
 			config->seen_mntoptions_require = true;
 		}
 		ret = str_to_optlist(config->options_require, options);
@@ -959,8 +963,7 @@ static int rc_volume_cond_uid(const struct passwd *pwd, xmlNode *node)
 			continue;
 		ret = __rc_volume_cond_id(signed_cast(const char *,
 		      node->content), pwd->pw_uid);
-		if (ret < 0)
-			return ret;
+		return ret;
 	}
 
 	l0g("config: empty or invalid content for <%s>\n", "uid");
@@ -981,8 +984,7 @@ static int rc_volume_cond_gid(const struct passwd *pwd, xmlNode *node)
 			continue;
 		ret = __rc_volume_cond_id(signed_cast(const char *,
 		      node->content), pwd->pw_gid);
-		if (ret < 0)
-			return ret;
+		return ret;
 	}
 
 	l0g("config: empty or invalid content for <%s>\n", "gid");
@@ -1327,6 +1329,9 @@ static const char *rc_volume(xmlNode *node, struct config *config,
 		vpt->fs_key_path = tmp;
 	}
 
+	if (fstype_nodev(vpt->fstype) == 1 && vpt->volume == NULL)
+		vpt->volume = xstrdup("none");
+
 	return NULL;
 
  out:
